/*
 * @(#)file      SyncManagerMultiV3.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.13
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// JDMK import
//
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.SnmpEngineId;

import com.sun.management.snmp.SnmpOidTableSupport;

/**
 * This class is a simple example showing how to use the SNMP V3 Manager 
 * API. It makes requests on 2 SNMP V3 agents.
 *
 * <p>When calling the program, you must specify:
 * <br>     - host1: hostname of the first SNMP agent you want to query.
 * <br>     - port1: First agent port number to use.
 * <br>     - host2: hostname of the second SNMP agent you want to query.
 * <br>     - port2: Second agent port number to use.
 * 
 * <p>As a reminder, if you wish to query the SNMP agent example provided
 *    as part of Java DMK, use ports 8085 and 8087.
 *
 * <p> A new Java DMK java property is used to set a security file :
 * <pre>
 *   -Djdmk.security.file=<i>file</i>
 * </pre>
 * <p>It contains security information related to users. Have a look at
 *    the READMEV3 file in order to find out how SNMP V3 managers are
 *    configured.
 */

public class SyncManagerMultiV3 {

    /**
     * Main entry point.
     * <p>When calling the program, you must specify:
     * <br>     - host1: hostname of the first SNMP agent you want to query.
     * <br>     - port1: First agent port number to use.
     * <br>     - host2: hostname of the second SNMP agent you want to query.
     * <br>     - port2: Second agent port number to use.
     * 
     * <p>As a reminder, if you wish to query the SNMP agent example provided
     *    as part of Java DMK, use ports 8085 and 8087.
     */
    public static void main(String argv[]) {
	SnmpSession session = null;
	
        if (argv.length != 4) {
            usage();
            java.lang.System.exit(1);
        }

	//Check arguments first
	//host and port.
        final String host1 = argv[0];
        final String port1 = argv[1];
	final String host2 = argv[2];
        final String port2 = argv[3];

        try {            
            // Initialize the SNMP Manager API.
            // Specify the OidTable containing all the MIB II knowledge.
            // Use the OidTable generated by mibgen when compiling MIB II.
            //
            final SnmpOidTableSupport oidTable = new RFC1213_MIBOidTable();
            SnmpOid.setSnmpOidTable(oidTable);
       
            // At any time, it is possible to add knowledge in the OidTable
            // using the SnmpOidDatabaseSupport.
            // For instance, we could start the Manager API like that:
            //    SnmpOidDatabaseSupport oidDB = 
	    //               new SnmpOidDatabaseSupport();
            //    SnmpOid.setSnmpOidTable(oidDB);
            // Then over time, MIB knowledge can be added as follows:
            //    oidDB.add(oidTable);
	  
	    // Build the session. A session creates, controls and manages 
	    // one or more requests.
            //
	    try {
		// When instantiating a session, a new SNMP V3 engine is 
		// instantiated.
		//
		session= new SnmpSession("SyncManagerMultiV3 session");
	    }catch(SnmpStatusException e) {
		println(e.getMessage());
		java.lang.System.exit(0);
	    }
	    catch(IllegalArgumentException e) {
		// If the engine configuration is faulty
		//
		println(e.getMessage());
		java.lang.System.exit(0);
	    }
	
	    // SNMP V3 introduces the notion of SnmpEngine. An engine is 
	    // associated to the session. Other objects  might need the 
	    // engine. You can access it using getEngine method.
	    //
	    final SnmpEngine engine = session.getEngine();
	    
	    // Create a SnmpPeer object for representing the first entity 
	    // to communicate with. 
	    // Java DMK 5.0 introduce a new SnmpPeer that is able to deal with 
	    // User based security parameters. Create it as a Java DMK 4.2 
	    // SnmpPeer and give it the local engine as additional parameter.
	    // A Usm peer discovers the engine Id of the SNMP V3 agent it 
	    // whishes to communicate with. This engine Id will be used 
	    // when sending requests.
	    //
	    final SnmpUsmPeer agent1 = 
		new SnmpUsmPeer(engine, host1, Integer.parseInt(port1));
	    
	    //Create the second peer.
	    //
	    final SnmpUsmPeer agent2 = 
		new SnmpUsmPeer(engine, host2, Integer.parseInt(port2));
	    
	    // Create parameters to associate to the entity to communicate 
	    // with.
	    // In our case, both agents are accessed using a common subset 
	    // of parameters.
	    // This parameter is used to parametrize SNMP requests.
	    // A user name is provided. This user must appears in the 
	    // jdmk.security file.
	    //
	    final SnmpUsmParameters p = 
		new SnmpUsmParameters(engine, "defaultUser");
	    
	    // Security level is set to authNoPriv. This means that 
	    // authentication is applied to the requests making use of 
	    // these parameters.
	    // This level of security must be in sync with the security 
	    // configuration located in jdmk.security. An authentication 
	    // password/algorithm must be provided. 
	    // If the security level is not in sync, an SnmpStatusException 
	    // with a failed status will be thrown when making requests.
	    //
	    p.setSecurityLevel(SnmpDefinitions.authNoPriv);

	    // In SNMP V3 you can register MIBS under the scope of a context.
	    // You set a context using setContextName(). In the purpose of 
	    // the example, the distant MIB is registered within the scope
	    // of TEST-CONTEXT.
	    //
	    p.setContextName("TEST-CONTEXT".getBytes());

	    // In SNMP V3 you can specify a contextEngineId. This is 
	    // usually the engine Id the agent discovered. Just reuse it.

	    // The interesting part. The engineId to use is linked to 
	    // the peer used when making the call. This parameter is 
	    // NOT common to both agent.
	    // This engineId MUST be changed in sync with the used peer.
	    // The first agent to be communicate with is agent1. We set 
	    // the engineId of agent1.
	    //
	    p.setContextEngineId(agent1.getEngineId().getBytes());
	    
	    // The newly created parameter must be associated to the agent.
	    //
	    agent1.setParams(p);
	
	    
	    // When making authenticated requests, some values must be 
	    // discovered.
	    // This is done calling processUsmTimelinessDiscovery.
	    // If this call is not done to the distant SNMP V3 agent, 
	    // authenticated requests will fail.
	    //
	    agent1.processUsmTimelinessDiscovery();
	    
	    
	    // Build the list of variables you want to query.
	    // For debug purposes, you can associate a name to your list.
	    //
	    final SnmpVarBindList list = 
		new SnmpVarBindList("SyncManagerMultiV3 varbind list");
	    
	    // We want to read the "sysDescr" variable.
	    //
            // We will thus query "sysDescr.0", as sysDescr is a scalar
	    // variable (see RFC 1157, section 3.2.6.3.  Identification 
            // of Object Instances, or RFC 2578, section 7.  Mapping of 
	    // the OBJECT-TYPE macro).
	    //
	    list.addVarBind("sysDescr.0");
	    
	    // Make the SNMP get request on the first agent agent1 
	    // and wait for the result.
	    //
	    SnmpRequest request = 
		session.snmpGetRequest(agent1, null, list);
	    println("SyncManagerMultiV3::main:" +
		    " Send get request to SNMP agent on " + 
		    host1 + " at port " + port1);
	    boolean completed = request.waitForCompletion(10000);
	    
	    // Check for a timeout of the request.
	    //
            if (completed == false) {
                println("SyncManagerMultiV3::main:" +
			" Request timed out. Check reachability of agent");
		
                // Print request.
                //
                println("Request: " + request.toString());
                java.lang.System.exit(0);
            }
	    
            // Now we have a response. Check if the response contains 
	    // an error.
            //
            int errorStatus = request.getErrorStatus();
            if (errorStatus != SnmpDefinitions.snmpRspNoError) {
                println("Error status = " + 
			SnmpRequest.snmpErrorToString(errorStatus));
                println("Error index = " + 
			request.getErrorIndex());
                java.lang.System.exit(0);
            }
       
            // Now we shall display the content of the result.
            //
            SnmpVarBindList result = request.getResponseVarBindList();
            println("Result: \n" + result);
       
            println("\n>> Press <Enter> if you want to send the request" +
		    " on the second agent.\n");
            java.lang.System.in.read();
            

	    //
	    // We do the same on the second agent agent2.
	    //
	    SnmpUsmParameters p2 = 
		new SnmpUsmParameters(engine, "defaultUser");
	    
	    p2.setSecurityLevel(SnmpDefinitions.authNoPriv);

	    p2.setContextName("TEST-CONTEXT".getBytes());
	    
	    p2.setContextEngineId(agent2.getEngineId().getBytes());
	    
	    // The updated parameters must be associated to the agent2.
	    //
	    agent2.setParams(p2);
	    	    
	    // When making authenticated requests, some values must be 
	    // discovered.
	    // This is done calling processUsmTimelinessDiscovery.
	    // If this call is not done to the distant SNMP V3 agent, 
	    // authenticated requests will fail.
	    //
	    agent2.processUsmTimelinessDiscovery();
	    
	    // Now we can make the request with agent2
	    //
	    request = session.snmpGetRequest(agent2, null, list);
	    println("SyncManagerMultiV3::main:" +
		    " Send get request to SNMP agent on " + 
		    host2 + " at port " + port2);
	    completed = request.waitForCompletion(10000);
	    
	    // Check for a timeout of the request.
	    //
            if (completed == false) {
                println("SyncManagerMultiV3::main:" +
			" Request timed out. Check reachability of agent");
		
                // Print request.
                //
                println("Request: " + request.toString());
                java.lang.System.exit(0);
            }
	    
            // Now we have a response. Check if the response contains 
	    // an error.
            //
	    errorStatus = request.getErrorStatus();
            if (errorStatus != SnmpDefinitions.snmpRspNoError) {
                println("Error status = " + 
			SnmpRequest.snmpErrorToString(errorStatus));
                println("Error index = " + 
			request.getErrorIndex());
                java.lang.System.exit(0);
            }
       
            // Now we shall display the content of the result.
            //
	    result = request.getResponseVarBindList();
            println("Result: \n" + result);
       
            println("\n>> Press <Enter> if you want to stop " +
		    "this manager.\n");
            java.lang.System.in.read();
	    
            // Nicely stop the session
            //
            session.destroySession();
       
            //
            // That's all !
            //
            java.lang.System.exit(0);
     
        } catch(Exception e) {
            java.lang.System.err.println("SyncManagerMultiV3::main: "+
					 "Exception occurred:" + e );
            e.printStackTrace();
        }
    }

    public static void  usage() {
        println("java -Djdmk.security.file=jdmk.security " +
		"SyncManagerMultiV3 <host1> <port1> <host2> <port2>");
        println("where");
        println("\t-host1: hostname or IP address of the first " +
		"SNMP agent you want to query.");
        println("\t-port1: First agent port number to use.");
	println("\t-host2: hostname or IP address of the second " +
		"SNMP agent you want to query.");
        println("\t-port3: Second agent port number to use.");
    }

    /**
     * print/println stuff...
     */

    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }

    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }
}
