/*
 * @(#)file      MasterAgent.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.15
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// Java imports
//
import java.io.IOException;
import java.net.SocketException;

// JMX imports
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

// JDMK imports
//
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.manager.SnmpEventReportDispatcher;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;

import com.sun.jdmk.comm.HtmlAdaptorServer;
import com.sun.management.comm.SnmpV3AdaptorServer;
import com.sun.management.snmp.agent.SnmpProxy;
import com.sun.management.snmp.agent.SnmpTrapForwarder;

/**
 * The MasterAgent class provides a simple example on how to use
 * the SnmpProxy.
 * <p>First launch the standalone agent. Then launch this master agent.
 * <p>Every snmp operations made on some OID implemented remotely
 * in the MIB instrumented by the distant agent will be forwarded.
 * <p>This agent is also a trap forwarder. It traps the standalone
 * agent traps and forwards them to its snmp trap port.
 */

public class MasterAgent {

    static SnmpV3AdaptorServer snmpAdaptor = null;

    /**
     * Main entry point.
     * When calling the program, you must specify:
     *  1) port: master agent port number.
     *  2) host: subagent hostname.
     *  3) port: subagent port number.
     */
    public static void main(String argv[]) {

        final MBeanServer server;
        final ObjectName htmlObjName;
        final ObjectName snmpObjName;
        final ObjectName mibObjName;
        final ObjectName proxyObjName;
        final ObjectName trapGeneratorObjName;
        final int htmlPort = 8082;

        if (argv.length != 3) {
            usage();
            java.lang.System.exit(1);
        }

        // Check arguments first.
        //
        final String myPort = argv[0];
        final String host = argv[1];
        final String port = argv[2];

        try {
            server = MBeanServerFactory.createMBeanServer();
            final String domain = server.getDefaultDomain();

            // Create and start the HTML adaptor.
            //
            htmlObjName = new ObjectName(domain +
                                         ":class=HtmlAdaptorServer," +
                                         "protocol=html,port=" + htmlPort);

            println("Adding HTML adaptor to MBean server with name \n\t" +
                    htmlObjName);
            println("NOTE: HTML adaptor is bound to TCP port " + htmlPort);

            final HtmlAdaptorServer htmlAdaptor =
                new HtmlAdaptorServer(htmlPort);
            server.registerMBean(htmlAdaptor, htmlObjName);
            htmlAdaptor.start();

            //
            // SNMP specific code:
            //

            // Create and start the SNMP adaptor.
            // Specify the port to use in the constructor.
            //
            snmpObjName = new ObjectName(domain +
                                         ":class=SnmpV3AdaptorServer" +
                                         ",protocol=snmp,port=" + myPort);
            println("Adding SNMP adaptor to MBean server with name \n\t" +
                    snmpObjName);
            println("NOTE: SNMP adaptor is bound to UDP port " + myPort);

            // Instantiates an SnmpV3AdaptorServer
            //
            snmpAdaptor = new SnmpV3AdaptorServer(Integer.parseInt(myPort));
            snmpAdaptor.setTrapPort(new Integer(Integer.parseInt(myPort)+1));
            println("The master agent forwards traps to port : " +
                    snmpAdaptor.getTrapPort());
            server.registerMBean(snmpAdaptor, snmpObjName);
            snmpAdaptor.start();

            //
            // MASTER AGENT FEATURE
            //

            // We now instantiate and register the SnmpProxy.
            // First we need to create a peer to the distant subagent.
            // We need to instantiate a SnmpPeer Java DMK manager API object.
            //
            final SnmpPeer peer = 
                new SnmpPeer(host, Integer.parseInt(port));
            final SnmpParameters p = new SnmpParameters();

            // The subagent is seen as an SNMP V2 one.
            //
            p.setProtocolVersion(SnmpDefinitions.snmpVersionTwo);

            // Set the parameters to the peer.
            //
            peer.setParams(p);
            
            // SnmpProxy creation.
            // We pass it :
            //    * an engine (the adaptor one).
            //    * a peer to communicate with the distant agent.
            //    * The root OID of the distant MIB.
            //
            final SnmpProxy proxy = new SnmpProxy(snmpAdaptor.getEngine(),
						  peer, "1.3.6.1.2.1");

            // A SnmpProxy is also an MBean. We register it in the 
            // MBeanServer.
            //
            proxyObjName= new ObjectName("snmp:class=SnmpProxy");
            server.registerMBean(proxy, proxyObjName);
            proxy.setSnmpAdaptorName(snmpObjName);

            //
            // TRAP FORWARDING FEATURE
            //
            
            // In order to forward traps, we instantiate a 
            // SnmpTrapForwarder providing it the adaptor engine.
            //
            final SnmpTrapForwarder trapForwarder = 
                new SnmpTrapForwarder(snmpAdaptor, 
                                      Integer.parseInt(port) + 1);

            // The traps will be also sent to the targets defined 
            // in the InetAddressAcl if one is loaded by the adaptor.
            // If no InetAddressAcl is loaded, the trap is forwarded to 
            // localhost.
            trapForwarder.setInetAddressAclUsed(true);

            trapForwarder.snmpV1forwarding(true);
            trapForwarder.snmpV2forwarding(true);

            // The SnmpTrapForwarder is an MBean, register it in the 
            // MBeanServer
            //
            final ObjectName n = 
                new ObjectName("master_agent:class=SnmpTrapForwarder,port=" 
                               + (Integer.parseInt(port) + 1));
            server.registerMBean(trapForwarder, n);

            // Start the forward. No specific configuration has been 
            // provided, it will forward to SNMP V1 and SNMP V2 managers 
            // only (thanks to IPAcl policy).
            //
            try {
                trapForwarder.start();
            } catch (SocketException e) { e.printStackTrace(); }
            
            
            println("\n>> Press <Enter> if you want to stop.");
            java.lang.System.in.read();
            java.lang.System.exit(0);
        
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void  usage() {
        println("java -Djdmk.security.file=jdmk.security MasterAgent " +
                "<master_agent_port> <subagent_host> <subagent_port>");
        println("where");
        println("\t-master_agent_port: master agent listening port");
        println("\t-subagent_host: hostname or IP address of the SNMP " +
                "standalone agent the master agent forwards to.");
        println("\t-subagent_port: standalone agent port number.");
    }

    /**
     * print/println stuff...
     */

    private final static void println(String msg) {
        java.lang.System.out.println(msg);
    }

    private final static void print(String msg) {
        java.lang.System.out.print(msg);
    }
}
