/*
 * @(#)file      Manager.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.9
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// JDMK imports
//
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;
import com.sun.management.snmp.manager.SnmpEventReportDispatcher;

import com.sun.management.snmp.SnmpOidTableSupport;
import com.sun.management.snmp.EnumRowStatus;

/**
 * The class is a simple example showing how to use the SNMP Manager API.
 *
 * It shows how to send a request that will create a row in a table that
 * follows the RowStatus convention defined in RFC 2579.
 *
 */

public class Manager {
   
    /**
     * Main entry point.
     * When calling the program, you must specify:
     *  1) host: hostname of the SNMP agent you want to query.
     *  2) port: port number to use.
     *
     * As a reminder, if you wish to query the SNMP agent example provided
     * as part of Java DMK, use port 8085.
     */
    public static void main(String argv[]) {
     
        if (argv.length != 2) {
            usage();
            java.lang.System.exit(1);
        }
    
        String host = argv[0];
        String port = argv[1];

        try {
            
            // Initialize the SNMP Manager API.
            // Specify the OidTable containing all the DEMO_MIB knowledge.
            // Use the OidTable generated by mibgen when compiling 
            // mib_demo.txt.
            //
            SnmpOidTableSupport oidTable = new DEMO_MIBOidTable();
            SnmpOid.setSnmpOidTable(oidTable);
       
            // At any time, it is possible to add knowledge in the OidTable
            // using the SnmpOidDatabaseSupport.
            // For instance, we could start the Manager API like that:
            //    SnmpOidDatabaseSupport oidDB = new SnmpOidDatabaseSupport();
            //    SnmpOid.setSnmpOidTable(oidDB);
            // Then over time, MIB knowledge can be added as follows:
            //    oidDB.add(oidTable);
            //
            
            // Create a SnmpPeer object for representing the entity to 
            // communicate with. 
            //
            SnmpPeer agent = new SnmpPeer(host, Integer.parseInt(port));
     
            // Create parameters to associate to the entity to 
            // communicate with.
            // When creating the parameter object, you can specify the read
            // and write community to
            // be used when querying the agent.
            // 
            SnmpParameters params = new SnmpParameters("public", "private");
       
            // The newly created parameter must be associated to the agent.
            //
            agent.setParams(params);
     
            // Build the session. A session creates, controls and manages 
            // one or more requests.
            //
            SnmpSession session= new SnmpSession("Manager session");
       
            // A default peer (agent) can be associated to a SnmpSession.
            // When invoking a service provided by the SnmpSession, if 
            // the agent is not specified, the session will perform
            // the service using the default peer as the target of the service.
            //
            session.setDefaultPeer(agent);
     

            // Declare temporary local variables
            //
            SnmpRequest     request     = null;
            SnmpVarBindList result      = null;
            boolean         completed   = false;
            int             errorStatus = 0;
            EnumRowStatus   createAndGo = new EnumRowStatus("createAndGo");
            EnumRowStatus   destroy     = new EnumRowStatus("destroy");


            // -------------------------------------------------
            //
            // Send a SET request in order to create an entry at index 1
            // in the demoTable.
            //
            // This is achieved by providing a VarBindList which contains
            // a "demoTableRowStatus" with value "createAndGo".
            //
            // -------------------------------------------------

            // Create the varbind list for the first SET request
            //
            SnmpVarBindList setList1 = 
                new SnmpVarBindList("Manager SET varbind list");
     
            // We want to create a row at index 1, so we have to give .1 
            // as row OID
            //
            SnmpVarBind stringRow1 = 
                new SnmpVarBind(new SnmpOid("demoTableString.1"),
                                new SnmpString("This is row 1"));
            SnmpVarBind statusRow1 = 
                new SnmpVarBind(new SnmpOid("demoTableRowStatus.1"),
                                createAndGo.toSnmpValue());

            setList1.addVarBind(stringRow1);
            setList1.addVarBind(statusRow1);
    
            // Make the SNMP get request and wait for the result.
            //
            java.lang.System.out.println("Manager::main:" + 
                      " Built SET request with RowStatus=createAndGo" + 
                      " to create new row at index 1.");
            request = session.snmpSetRequest(null, setList1);
            java.lang.System.out.println("Manager::main:" + 
                                         " Send SET request to SNMP agent" +
                                         " on " + host + " at port " + port);
            completed = request.waitForCompletion(10000);
            
            // Check for a timeout of the request.
            //
            if (completed == false) {
                java.lang.System.out.println("Manager::main:" + 
                                             " SET Request timed out." + 
                                             " Check reachability of agent");
         
                // Print request.
                //
                java.lang.System.out.println("SET Request: " +
					     request.toString());
                java.lang.System.exit(0);
            }
     
            // Now we have a response. Check if the response contains an error.
            //
            errorStatus = request.getErrorStatus();
            if (errorStatus != SnmpDefinitions.snmpRspNoError) {
                java.lang.System.out.println("Error status = " + 
                                  SnmpRequest.snmpErrorToString(errorStatus));
                java.lang.System.out.println("Error index = " + 
                                             request.getErrorIndex());
                java.lang.System.exit(0);
            }
       
            // Now we shall display the content of the result.
            //
            result = request.getResponseVarBindList();
            java.lang.System.out.println("SET Result: \n" + result);
      

            java.lang.System.out.println("\n>> Press <Enter> to proceed " +
					 "with the GET request.\n");
            java.lang.System.in.read();

            // -------------------------------------------------
            //
            // Send a GET request in order to check that the entry at 
            // index 1 was correctly created.
            //
            // We're going to query:
            //
            //    demoInteger = # of rows in the table (see DemoImpl.java)
            //
            //    demoTableString.1 = value of demoTableString at index 1
            //
            //    demoTableRowStatus.1 = value of RowStatus for row at index 1
            //
            // -------------------------------------------------

            // Build the list of variables you want to query.
            // For debug purposes, you can associate a name to your list.
            //
            SnmpVarBindList getList = 
                new SnmpVarBindList("Manager GET varbind list");
     
            // We want to read the number of instances in the table
            // ("demoInteger") and get the "demoTableString" and 
            // "demoTableRowStatus" variables of the entry located at index 1.
            //
            getList.addVarBind("demoInteger.0");
            getList.addVarBind("demoTableString.1");
            getList.addVarBind("demoTableRowStatus.1");
    
            // Make the SNMP get request and wait for the result.
            //
            java.lang.System.out.println("Manager::main:" +
                      " Built GET request to check the presence of new row" +
                      " at index 1");
            request = session.snmpGetRequest(null, getList);
            java.lang.System.out.println("Manager::main:" + 
                                         " Send GET request to SNMP agent" +
                                         " on " + host + " at port " + port);
            completed = request.waitForCompletion(10000);
            
            // Check for a timeout of the request.
            //
            if (completed == false) {
                java.lang.System.out.println("Manager::main:" + 
                                             " GET Request timed out." + 
                                             " Check reachability of agent");
         
                // Print request.
                //
                java.lang.System.out.println("GET Request: " + 
                                             request.toString());
                java.lang.System.exit(0);
            }
     
            // Now we have a response. Check if the response contains an error.
            //
            errorStatus = request.getErrorStatus();
            if (errorStatus != SnmpDefinitions.snmpRspNoError) {
                java.lang.System.out.println("Error status = " + 
                                  SnmpRequest.snmpErrorToString(errorStatus));
                java.lang.System.out.println("Error index = " + 
                                             request.getErrorIndex());
                java.lang.System.exit(0);
            }
       
            // Now we shall display the content of the result.
            //
            result = request.getResponseVarBindList();
            java.lang.System.out.println("GET Result: \n" + result);
      

            java.lang.System.out.println("\n>> Press <Enter> to proceed with " +
					 "the second SET request.\n");
            java.lang.System.in.read();

            // -------------------------------------------------
            //
            // Send a SET request in order to remove the entry at index 1
            // in the demoTable.
            //
            // This is achieved by providing a VarBindList which contains
            // a "demoTableRowStatus" with value "destroy".
            //
            // -------------------------------------------------

            // Create the varbind list for the first SET request
            //
            SnmpVarBindList setList2 = 
                new SnmpVarBindList("Manager SET varbind list");
     
            // We want to destroy the row at index 1
            //
            SnmpVarBind statusRow2 = 
                new SnmpVarBind(new SnmpOid("demoTableRowStatus.1"),
                                destroy.toSnmpValue());

            setList2.addVarBind(statusRow2);
    
            // Make the SNMP get request and wait for the result.
            //
            java.lang.System.out.println("Manager::main:" + 
                      " Built SET request with RowStatus=destroy" + 
                      " to remove row at index 1.");
            request = session.snmpSetRequest(null, setList2);
            java.lang.System.out.println("Manager::main:" + 
                                         " Send SET request to SNMP agent" +
                                         " on " + host + " at port " + port);
            completed = request.waitForCompletion(10000);
            
            // Check for a timeout of the request.
            //
            if (completed == false) {
                java.lang.System.out.println("Manager::main:" + 
                                             " SET Request timed out." + 
                                             " Check reachability of agent");
         
                // Print request.
                //
                java.lang.System.out.println("SET Request: " + request.toString());
                java.lang.System.exit(0);
            }
     
            // Now we have a response. Check if the response contains an error.
            //
            errorStatus = request.getErrorStatus();
            if (errorStatus != SnmpDefinitions.snmpRspNoError) {
                java.lang.System.out.println("Error status = " + 
                                  SnmpRequest.snmpErrorToString(errorStatus));
                java.lang.System.out.println("Error index = " + 
                                             request.getErrorIndex());
                java.lang.System.exit(0);
            }
       
            // Now we shall display the content of the result.
            //
            result = request.getResponseVarBindList();
            java.lang.System.out.println("SET Result: \n" + result);
      

            java.lang.System.out.println("\n>> Press <Enter> to proceed with " +
					 "the second GET request.\n");
            java.lang.System.in.read();

            // -------------------------------------------------
            //
            // Send a GET request in order to check that the entry at 
            // index 1 was correctly created.
            //
            // We're going to query:
            //
            //    demoInteger = # of rows in the table (see DemoImpl.java)
            //
            //    demoTableString.1 = value of demoTableString at index 1
            //                        (Should be "noSuchInstance")
            //
            //    demoTableRowStatus.1 = value of RowStatus for row at index 1
            //                        (Should be "noSuchInstance")
            //
            // We will reuse the varbindlist we built for the first
            // GET request.
            //
            // -------------------------------------------------

            // Make the SNMP get request and wait for the result.
            //
            java.lang.System.out.println("Manager::main:" +
                      " Built GET request to check that row at index 1" +
                      " is no longer there.");
            request = session.snmpGetRequest(null, getList);
            java.lang.System.out.println("Manager::main:" + 
                                         " Send GET request to SNMP agent" +
                                         " on " + host + " at port " + port);
            completed = request.waitForCompletion(10000);
            
            // Check for a timeout of the request.
            //
            if (completed == false) {
                java.lang.System.out.println("Manager::main:" + 
                                             " GET Request timed out." + 
                                             " Check reachability of agent");
         
                // Print request.
                //
                java.lang.System.out.println("GET Request: " + 
                                             request.toString());
                java.lang.System.exit(0);
            }
     
            // Now we have a response. Check if the response contains an error.
            //
            errorStatus = request.getErrorStatus();
            if (errorStatus != SnmpDefinitions.snmpRspNoError) {
                java.lang.System.out.println("Error status = " + 
                                  SnmpRequest.snmpErrorToString(errorStatus));
                java.lang.System.out.println("Error index = " + 
                                             request.getErrorIndex());
            }
                
            // Now we shall display the content of the result.
            //
            result = request.getResponseVarBindList();
            java.lang.System.out.println("GET Result: \n" + result);


            java.lang.System.out.println("\n>> Press <Enter> if you want to " +
					 "stop this SNMP manager.\n");
            java.lang.System.in.read();
            
            // Nicely stop the session
            //
            session.destroySession();
       
            //
            // That's all !
            //
            java.lang.System.exit(0);
     
        } catch (Exception e) {
            java.lang.System.err.println("Manager::main: Exception occurred:" +
					 e );
            e.printStackTrace();
        }
    }
   
    /**
     * Return usage of the program.
     */
    public static void  usage() {
        java.lang.System.out.println("java Manager <host> <port>");
        java.lang.System.out.println("where");
        java.lang.System.out.println("\t-host: hostname or IP address of the " +
				     "SNMP agent you want to query.");
        java.lang.System.out.println("\t-port: port number to use.");
    }
}
