/*
 * @(#)file      Agent.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)lastedit  04/04/21
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// JMX imports
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

// JDMK imports
//
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpEngineParameters;

import com.sun.jdmk.comm.HtmlAdaptorServer;
import com.sun.management.comm.SnmpV3AdaptorServer;
import com.sun.management.comm.SnmpAdaptorServer;

/**
 * The Agent class provides a simple example on how to use the
 * SNMP V3 protocol adaptor, with encryption activated.
 * This Agent registers the USM MIB and wait for a Manager to create
 * new USM users.
 *
 * <p>Configuration is updated in jdmk.security file.
 *
 * <p>In order to check that new users have been added in the USM
 *    configuration, you can use the HTML adaptor.
 *
 * <p>In this example, the SNMP adaptor is started on port 8085,
 *    and the traps are sent to the port 8086, i.e non standard
 *    ports for SNMP. This avoids the necessity to start the agent
 *    as super-user, as the regular SNMP ports (161 and 162) require
 *    privileged access.
 */

public class Agent {

    static SnmpV3AdaptorServer snmpAdaptor = null;

    /**
     * Main entry point.
     */
    public static void main(String args[]) {
        
        final MBeanServer server;
        final ObjectName htmlObjName;
        final ObjectName snmpObjName;
        final int htmlPort = 8082;
        int snmpPort = 161;

        try {
            server = MBeanServerFactory.createMBeanServer();
            final String domain = server.getDefaultDomain();

            // Create and start the HTML adaptor.
            //
            htmlObjName = new ObjectName(domain +
                  ":class=HtmlAdaptorServer,protocol=html,port=" + htmlPort);
            println("Adding HTML adaptor to MBean server with name \n\t" +
                    htmlObjName);
            println("NOTE: HTML adaptor is bound to TCP port " + htmlPort);
            final HtmlAdaptorServer htmlAdaptor =
                new HtmlAdaptorServer(htmlPort);
            server.registerMBean(htmlAdaptor, htmlObjName);
            htmlAdaptor.start();

            //
            // SNMP specific code:
            //

            // Create and start the SNMP adaptor.
            // Specify the port to use in the constructor.
            // If you want to use the standard port (161)
	    // comment out the line:
	    //
            // snmpPort = 8085;
            //
            snmpPort = 8085;
            snmpObjName = new ObjectName(domain +
                  ":class=SnmpAdaptorServer,protocol=snmp,port=" + snmpPort);
            println("Adding SNMP adaptor to MBean server with name \n\t" +
                    snmpObjName);
            println("NOTE: SNMP Adaptor is bound to UDP port " + snmpPort);

            // Handling the encryption activation.
            //
            // By default, Java DMK Agent API handles unsecured an
            // authenticated snmp requests. Additionaly, privacy can be
            // activated. In this case, some parameters need to be set
            // in order to activate this additional level of security. 
            // This is done thanks to a new Java DMK class called
            // SnmpEngineParemeters.
            // If you do not need to activate encryption or provide
            // any specific parameters, the you will not need to use
            // this class.

            // First create parameters.
            //
            final SnmpEngineParameters parameters =
                new SnmpEngineParameters();

            // Then activate encryption
            //
            parameters.activateEncryption();

            // Finaly create the adaptor passing it the parameters.
            snmpAdaptor =
                new SnmpV3AdaptorServer(parameters,null,null,snmpPort,null);

            server.registerMBean(snmpAdaptor, snmpObjName);

            // Register the USM MIB in the adaptor.
            // This makes the USM MIB accessible to remote manager.
            // Passing the MBeanServer also makes the MBeans representing
            // the MIB being registered as MBeans in the MBeanServer.
            //
            snmpAdaptor.registerUsmMib(server, null);
            snmpAdaptor.start();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * print/println stuff...
     */

    private final static void println(String msg) {
        java.lang.System.out.println(msg);
    }

    private final static void print(String msg) {
        java.lang.System.out.print(msg);
    }
}
